/*
 * Process spikey packet logs into bits per second over time.
 * Input in a file of time in floating point seconds and a number of
 * Bytes in a PDU recorded at that time.  Assumes that the transmission
 * rate for that PDU was essentially instantaneous at the recorded second.
 * This computes bits per second by totaling all PDU bytes in a given
 * interval and dividing by the interval length.
 *
 * phil@arl.mil
 */
#include <stdio.h>

int abs_start = 0;
int start_time = 0;		/* seconds */
int interval = 60;		/* seconds */

int dohours = 0;
int dodis = 0;

static char usage[] = "\
Usage: doit [-options] [interval] < file\n\
 -h  output time in hours (seconds is the default)\n\
 -d  input is DIS PDU's (adds packet overhead)\n\
";

main(argc, argv)
int argc;
char **argv;
{
	FILE	*fp;
	double	time;
	int	len;
	int	total;
	double	bps;
	int	plot_time;

	while (argc > 1) {
		if (strcmp(argv[1], "-h") == 0) {
			dohours = 1;
		} else if (strcmp(argv[1], "-d") == 0) {
			dodis = 1;
		} else
			break;
		argc--;
		argv++;
	}
	if (argc == 2) {
		interval = atoi(argv[1]);
	}
	if (interval == 0 || argc > 2) {
		fprintf(stderr, usage);
		exit(1);
	}

	total = 0;
#if 0
	fp = fopen("process.out", "r");
#else
	fp = stdin;
#endif
	if (fp == NULL) {
		fprintf(stderr, "can't open process.out");
		exit(1);
	}
	if (isatty(fileno(fp))) {
		fprintf(stderr, usage);
		exit(1);
	}

	while (!feof(fp)) {
		fscanf(fp, "%lf %d\n", &time, &len);
		/*printf("%f %d\n", time, len);*/

		if (dodis)
			len += 28;	/* IP + UDP header bytes */

		/* start first interval at first PDU */
		if (start_time == 0) {
			start_time = time;
		}

		if (time < start_time)
			/* PDU before start of interval */
			continue;
		if (time < start_time+interval) {
			/* PDU in the interval */
			total += len;
		} else {
			/* PDU past end of interval */
			if (total <= 0)
				bps = 0;
			else
				bps = (total*8)/(double)interval;
			plot_time = start_time+interval/2 - abs_start;
			if (dohours)
				printf("%f %f\n", plot_time/3600.0, bps);
			else
				printf("%d %f\n", plot_time, bps);
			total = 0;
			start_time += interval;
			/* add this PDU to next interval? XXX */
			if (time < start_time+interval) {
				total += len;
			}
		}
	}
	fclose(fp);
}
